from django.test import TestCase
from cradmin_legacy.utils import crhumanize


class TestHumanReadableFileSize(TestCase):
    def test_less_than_kb(self):
        self.assertEqual("0B", crhumanize.human_readable_filesize(0))
        self.assertEqual("999B", crhumanize.human_readable_filesize(999))

    def test_kb(self):
        self.assertEqual("1KB", crhumanize.human_readable_filesize(1000))
        self.assertEqual("999KB", crhumanize.human_readable_filesize(999999))

    def test_mb(self):
        self.assertEqual("1.0MB", crhumanize.human_readable_filesize(1000000))
        self.assertEqual("55.6MB", crhumanize.human_readable_filesize(55555555))

    def test_gb(self):
        self.assertEqual("1.00GB", crhumanize.human_readable_filesize(1000000000))
        self.assertEqual("55.56GB", crhumanize.human_readable_filesize(55555555555))

    def test_tb(self):
        self.assertEqual("1.00TB", crhumanize.human_readable_filesize(1000000000000))
        self.assertEqual("55.56TB", crhumanize.human_readable_filesize(55555555555555))


class TestDehumanizeReadableFileSize(TestCase):
    def test_less_than_kb(self):
        self.assertEqual(0, crhumanize.dehumanize_readable_filesize("0B"))
        self.assertEqual(999, crhumanize.dehumanize_readable_filesize("999B"))

    def test_kb(self):
        self.assertEqual(1000, crhumanize.dehumanize_readable_filesize("1KB"))
        self.assertEqual(999000, crhumanize.dehumanize_readable_filesize("999KB"))

    def test_mb(self):
        self.assertEqual(1000000, crhumanize.dehumanize_readable_filesize("1.0MB"))
        self.assertEqual(55600000, crhumanize.dehumanize_readable_filesize("55.6MB"))

    def test_gb(self):
        self.assertEqual(1000000000, crhumanize.dehumanize_readable_filesize("1.00GB"))
        self.assertEqual(55560000000, crhumanize.dehumanize_readable_filesize("55.56GB"))

    def test_tb(self):
        self.assertEqual(1000000000000, crhumanize.dehumanize_readable_filesize("1.00TB"))
        self.assertEqual(55560000000000, crhumanize.dehumanize_readable_filesize("55.56TB"))



from django.test import TestCase
from model_bakery import baker

from cradmin_legacy.tests.viewhelpers.cradmin_viewhelpers_testapp.models import FilterTestModel
from cradmin_legacy.viewhelpers import listfilter


class TestSearch(TestCase):
    def test_no_value(self):
        testfilter = listfilter.django.single.textinput.Search(
            slug="mycharfield", modelfields=["mycharfield", "mytextfield"]
        )
        testfilter.set_values(values=[])
        withvalue1 = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="A testvalue")
        withvalue2 = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mytextfield="Another testvalue")
        emptyvalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="")
        nullvalue = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield=None)
        self.assertEqual(
            {withvalue1, withvalue2, emptyvalue, nullvalue},
            set(testfilter.filter(queryobject=FilterTestModel.objects.all())),
        )

    def test_matching_single(self):
        testfilter = listfilter.django.single.textinput.Search(
            slug="mycharfield", modelfields=["mycharfield", "mytextfield"]
        )
        testfilter.set_values(values=["Another"])
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="A testvalue")
        withvalue2 = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mytextfield="Another testvalue")
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="")
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield=None)
        self.assertEqual({withvalue2}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))

    def test_matching_multiple(self):
        testfilter = listfilter.django.single.textinput.Search(
            slug="mycharfield", modelfields=["mycharfield", "mytextfield"]
        )
        testfilter.set_values(values=["testvalue"])
        withvalue1 = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="A testvalue")
        withvalue2 = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mytextfield="Another testvalue")
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="")
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield=None)
        self.assertEqual({withvalue1, withvalue2}, set(testfilter.filter(queryobject=FilterTestModel.objects.all())))



import htmls
from django.test import TestCase

from cradmin_legacy.python2_compatibility import mock
from cradmin_legacy.viewhelpers.listfilter.base.abstractfilter import AbstractFilter
from cradmin_legacy.viewhelpers.listfilter.base.abstractfilterlist import AbstractFilterList


class MinimalIntFilter(AbstractFilter):
    template_name = "cradmin_viewhelpers_testapp/listfilter/minimal-filtergroupchild.django.html"

    def get_slug(self):
        return "i"

    def clean_value(self, value):
        return int(value)


class MinimalStringFilter(AbstractFilter):
    template_name = "cradmin_viewhelpers_testapp/listfilter/minimal-filtergroupchild.django.html"

    def get_slug(self):
        return "s"


class TestAbstractFilter(TestCase):
    def test_copy_include_values(self):
        testfilter = MinimalStringFilter()
        testfilter.set_values(values=["a", "b"])
        self.assertEqual(["a", "b"], testfilter.copy().values)

    def test_copy_include_values_does_not_affect_original(self):
        testfilter = MinimalStringFilter()
        testfilter.set_values(values=["a", "b"])
        copyfilter = testfilter.copy()
        copyfilter.values[0] = "x"
        self.assertEqual(["a", "b"], testfilter.values)
        self.assertEqual(["x", "b"], copyfilter.values)

    def test_copy_include_slug(self):
        testfilter = MinimalStringFilter(slug="testslug")
        self.assertEqual("testslug", testfilter.copy().slug)

    def test_copy_include_label(self):
        testfilter = MinimalStringFilter(label="testlabel")
        self.assertEqual("testlabel", testfilter.copy().label)

    def test_set_values(self):
        testfilter = MinimalStringFilter()
        self.assertEqual([], testfilter.values)
        testfilter.set_values(values=["a", "b"])
        self.assertEqual(["a", "b"], testfilter.values)

    def test_clear_values(self):
        testfilter = MinimalStringFilter()
        testfilter.values = ["a"]
        testfilter.clear_values()
        self.assertEqual([], testfilter.values)

    def test_add_values(self):
        testfilter = MinimalStringFilter()
        testfilter.values = ["a", "b"]
        testfilter.add_values(values=["c", "d"])
        self.assertEqual(["a", "b", "c", "d"], testfilter.values)

    def test_remove_values(self):
        testfilter = MinimalStringFilter()
        testfilter.values = ["a", "b", "c", "d"]
        testfilter.remove_values(values=["a", "d"])
        self.assertEqual(["b", "c"], testfilter.values)

    def test_remove_values_ignores_nonexisting_values(self):
        testfilter = MinimalStringFilter()
        testfilter.values = ["a", "b"]
        testfilter.remove_values(values=["doesnotexist", "b"])
        self.assertEqual(["a"], testfilter.values)

    def test_get_cleaned_values(self):
        class SimpleFilter(AbstractFilter):
            def clean_value(self, value):
                return "cleaned-" + value

        testfilter = SimpleFilter()
        testfilter.values = ["a", "b"]
        self.assertEqual(["cleaned-a", "cleaned-b"], testfilter.get_cleaned_values())

    def test_build_add_values_url(self):
        stringfilter = MinimalStringFilter()
        stringfilter.set_values(values=["a", "b"])
        intfilter = MinimalIntFilter()
        intfilter.set_values(values=["10"])
        filterlist = AbstractFilterList(
            urlbuilder=lambda filters_string: f"/test/{filters_string}", target_dom_id="testdomid"
        )
        filterlist.append(stringfilter)
        filterlist.append(intfilter)
        stringfilter.set_filterlist(filterlist)
        self.assertEqual("/test/s-a%2Cb%2Cc/i-10", stringfilter.build_add_values_url(values=["c"]))

    def test_build_clear_values_url(self):
        stringfilter = MinimalStringFilter()
        stringfilter.set_values(values=["a", "b"])
        intfilter = MinimalIntFilter()
        intfilter.set_values(values=["10"])
        filterlist = AbstractFilterList(
            urlbuilder=lambda filters_string: f"/test/{filters_string}", target_dom_id="testdomid"
        )
        filterlist.append(stringfilter)
        filterlist.append(intfilter)
        stringfilter.set_filterlist(filterlist)
        self.assertEqual("/test/i-10", stringfilter.build_clear_values_url())

    def test_build_set_values_url(self):
        stringfilter = MinimalStringFilter()
        stringfilter.set_values(values=["a", "b"])
        intfilter = MinimalIntFilter()
        intfilter.set_values(values=["10"])
        filterlist = AbstractFilterList(
            urlbuilder=lambda filters_string: f"/test/{filters_string}", target_dom_id="testdomid"
        )
        filterlist.append(stringfilter)
        filterlist.append(intfilter)
        stringfilter.set_filterlist(filterlist)
        self.assertEqual("/test/s-c/i-10", stringfilter.build_set_values_url(values=["c"]))

    def test_build_remove_values_url(self):
        stringfilter = MinimalStringFilter()
        stringfilter.set_values(values=["a", "b"])
        intfilter = MinimalIntFilter()
        intfilter.set_values(values=["10"])
        filterlist = AbstractFilterList(
            urlbuilder=lambda filters_string: f"/test/{filters_string}", target_dom_id="testdomid"
        )
        filterlist.append(stringfilter)
        filterlist.append(intfilter)
        stringfilter.set_filterlist(filterlist)
        self.assertEqual("/test/s-b/i-10", stringfilter.build_remove_values_url(values=["a"]))

    def test_render(self):
        stringfilter = AbstractFilter(slug="test")
        stringfilter.set_filterlist(mock.MagicMock())
        selector = htmls.S(stringfilter.render())
        self.assertTrue(selector.exists(".cradmin-legacy-listfilter-filter"))



from datetime import timedelta, datetime
from django.contrib import messages
from django.contrib.auth import get_user_model
from django.urls import reverse
from django.test import TestCase, RequestFactory
from django.utils import timezone
import htmls
from cradmin_legacy.python2_compatibility import mock
from cradmin_legacy.apps.cradmin_resetpassword.views.reset import ResetPasswordView
from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata
from cradmin_legacy.tests.helpers import create_user


class TestResetPasswordView(TestCase):
    def setUp(self):
        self.testuser = create_user("testuser")

    def __get_url(self, token):
        return reverse("cradmin-resetpassword-reset", kwargs={"token": token})

    def _create_generic_token_with_metadata(self, user, created_datetime=None, expiration_datetime=None, **kwargs):
        generic_token_with_metadata = GenericTokenWithMetadata.objects.create(
            created_datetime=(created_datetime or timezone.now()),
            expiration_datetime=(expiration_datetime or (timezone.now() + timedelta(days=2))),
            content_object=user,
            app="cradmin_resetpassword",
            **kwargs,
        )
        return generic_token_with_metadata

    def test_get(self):
        self._create_generic_token_with_metadata(token="valid-token", user=self.testuser)
        response = self.client.get(self.__get_url("valid-token"))
        selector = htmls.S(response.content)
        self.assertEqual(selector.one("h1").alltext_normalized, "Reset your password testuser")
        self.assertTrue(selector.exists("form#cradmin_legacy_resetpassword_reset_form"))
        self.assertTrue(selector.exists('input[type="password"][name="password1"]'))
        self.assertTrue(selector.exists('input[type="password"][name="password2"]'))

    def test_get_expired_token(self):
        self._create_generic_token_with_metadata(
            token="valid-token", user=self.testuser, expiration_datetime=datetime(2014, 1, 1)
        )
        response = self.client.get(self.__get_url("valid-token"))
        selector = htmls.S(response.content)
        self.assertFalse(selector.exists("form#cradmin_legacy_resetpassword_reset_form"))
        self.assertTrue(selector.exists("#cradmin_legacy_resetpassword_reset_expired_message"))
        self.assertEqual(
            selector.one("#cradmin_legacy_resetpassword_reset_expired_message").alltext_normalized,
            "This password reset link has expired.",
        )

    def test_get_invalid_token(self):
        self._create_generic_token_with_metadata(token="valid-token", user=self.testuser)
        response = self.client.get(self.__get_url("invalid-token"))
        selector = htmls.S(response.content)
        self.assertFalse(selector.exists("form#cradmin_legacy_resetpassword_reset_form"))
        self.assertTrue(selector.exists("#cradmin_legacy_resetpassword_reset_invalid_token_message"))
        self.assertEqual(
            selector.one("#cradmin_legacy_resetpassword_reset_invalid_token_message").alltext_normalized,
            "Invalid password reset URL. Are you sure you copied the entire URL from the email?",
        )

    def test_post_passwords_not_matching(self):
        self._create_generic_token_with_metadata(token="valid-token", user=self.testuser)
        response = self.client.post(
            self.__get_url("valid-token"),
            {
                "password1": "passwordOne",
                "password2": "passwordTwo",
            },
        )
        selector = htmls.S(response.content)
        self.assertIn(
            "The passwords do not match",
            selector.one("form#cradmin_legacy_resetpassword_reset_form").alltext_normalized,
        )

    def test_post_expired_token(self):
        self._create_generic_token_with_metadata(
            token="valid-token", user=self.testuser, expiration_datetime=datetime(2014, 1, 1)
        )
        response = self.client.post(
            self.__get_url("valid-token"),
            {
                "password1": "secret",
                "password2": "secret",
            },
        )
        selector = htmls.S(response.content)
        self.assertFalse(selector.exists("form#cradmin_legacy_resetpassword_reset_form"))
        self.assertTrue(selector.exists("#cradmin_legacy_resetpassword_reset_expired_message"))
        self.assertEqual(
            selector.one("#cradmin_legacy_resetpassword_reset_expired_message").alltext_normalized,
            "This password reset link has expired.",
        )

    def test_post_invalid_token(self):
        self._create_generic_token_with_metadata(token="valid-token", user=self.testuser)
        response = self.client.post(
            self.__get_url("invalid-token"),
            {
                "password1": "secret",
                "password2": "secret",
            },
        )
        selector = htmls.S(response.content)
        self.assertFalse(selector.exists("form#cradmin_legacy_resetpassword_reset_form"))
        self.assertTrue(selector.exists("#cradmin_legacy_resetpassword_reset_invalid_token_message"))
        self.assertEqual(
            selector.one("#cradmin_legacy_resetpassword_reset_invalid_token_message").alltext_normalized,
            "Invalid password reset URL. Are you sure you copied the entire URL from the email?",
        )

    def test_post(self):
        self._create_generic_token_with_metadata(token="valid-token", user=self.testuser)

        with self.settings(CRADMIN_LEGACY_RESETPASSWORD_FINISHED_REDIRECT_URL="http://example.com"):
            response = self.client.post(
                self.__get_url("valid-token"),
                {
                    "password1": "newpassword",
                    "password2": "newpassword",
                },
            )
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response["location"], "http://example.com")
        testuser = get_user_model().objects.get(pk=self.testuser.pk)
        self.assertTrue(testuser.check_password("newpassword"))

    def test_post_success_message(self):
        self._create_generic_token_with_metadata(token="valid-token", user=self.testuser)

        request = RequestFactory().post(
            "/test",
            {
                "password1": "newpassword",
                "password2": "newpassword",
            },
        )
        request.user = self.testuser
        request._messages = mock.MagicMock()

        with self.settings(CRADMIN_LEGACY_RESETPASSWORD_FINISHED_REDIRECT_URL="http://example.com"):
            ResetPasswordView.as_view()(request, token="valid-token")
        request._messages.add.assert_called_once_with(messages.SUCCESS, "Your password has been updated.", "")



import logging
import mimetypes
from sorl.thumbnail import get_thumbnail
from cradmin_legacy.imageutils.backends import backendinterface


logger = logging.getLogger(__name__)


class SorlThumbnail(backendinterface.Interface):
    """
    Sorl-thumbnail backend for ``cradmin_legacy.imageutils``.
    """

    def transform_image(self, imageurl, **options):
        width = options.get("width", None)
        height = options.get("height", None)
        quality = options.get("quality", 100)
        crop = options.get("crop", "limit")

        if not width and not height:
            raise ValueError("At least one of width and height must be specified")
        width = width or ""
        height = height or ""
        size = f"{width}x{height}"

        if crop == "limit":
            upscale = False
            sorl_crop = None
        else:
            upscale = True
            sorl_crop = "center"

        mimetype = mimetypes.guess_type(imageurl)[0]
        if mimetype == "image/jpeg":
            imageformat = "JPEG"
        elif mimetype == "image/png":
            imageformat = "PNG"
        elif mimetype == "image/gif":
            imageformat = "GIF"
        else:
            imageformat = "JPEG"

        try:
            thumbnail = get_thumbnail(
                imageurl, size, upscale=upscale, quality=quality, crop=sorl_crop, format=imageformat
            )
            return thumbnail.url
        except Exception as e:
            logger.exception(e)
            return ""
